(*
 * Generate magic numbers.
 *
 * ----------------------------------------------------------------
 *
 * @begin[license]
 * Copyright (C) 2003 Jason Hickey, Caltech
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Jason Hickey
 * @email{jyh@cs.caltech.edu}
 * @end[license]
 *)
open Lm_printf

(*
 * Get the version from the version.txt file.
 *)
let rec read_version_from_file inx =
   let line = Lm_string_util.trim (input_line inx) in
      if line = "" then
         raise End_of_file;
      line

let read_version () =
   let file = "version.txt" in
   let inx = open_in file in
   let version =
      try read_version_from_file inx with
         End_of_file ->
            eprintf "The %s file does not contain a version number@." file;
            exit 1
   in
      close_in inx;
      version

(*
 * Copy a file from input to output.
 *)
let rec copy_file buf inx =
   let line =
      try Some (input_line inx) with
         End_of_file ->
            None
   in
      match line with
         Some line ->
            (* Translate the open lines *)
            let len = String.length line in
            let line =
               if len > 2 && String.sub line 0 2 = "#+" then
                  String.sub line 2 (len - 2)
               else if len > 7 && String.sub line 0 7 = "include" then
                  ""
               else
                  line
            in
               fprintf buf "%s@ " line;
               copy_file buf inx
       | None ->
            ()

(************************************************************************
 * omake_magic.ml file.
 *)

(*
 * Save the date in the version string.
 *)
let wday_names =
   [|"Sun"; "Mon"; "Tue"; "Wed"; "Thu"; "Fri"; "Sat"|]

let mon_names =
   [|"Jan"; "Feb"; "Mar"; "Apr"; "May"; "Jun"; "Jul"; "Aug"; "Sep"; "Oct"; "Nov"; "Dec"|]

let cache_files =
   ".odb",
   ["omake_cache.ml";
    "omake_cache_type.ml";
    "omake_node_type.ml"]

let env_files =
   ".oen",
   ["omake_build_type.ml"]

let ir_files =
   ".omc",
   ["omake_env.mli";
    "omake_ir.ml"]

let obj_files =
   ".omo",
   ["omake_env.mli";
    "omake_lexer.ml";
    "omake_parser.ml"]

let digest_files (pref, files) =
   let s =
      match files with
         [file] ->
            pref ^ Digest.file file
       | _ ->
            let s =
               List.fold_left (fun s file ->
                     Digest.file file ^ s) "digest" files
            in
               pref ^ Digest.string s
   in
      String.escaped s

let digest_len = 4 + String.length (Digest.string "hello world")

let omake_magic buf =
   let s = Sys.getenv "LIBDIR" in
   let s = Filename.concat s "omake" in
   let version = read_version () in
   let now = Unix.time () in
   let { Unix.tm_year = year;
         Unix.tm_mon = mon;
         Unix.tm_mday = mday;
         Unix.tm_wday = wday;
         Unix.tm_hour = hour;
         Unix.tm_min = min;
         Unix.tm_sec = sec
       } = Unix.localtime now
   in
      fprintf buf "let input_magic inx = let s = String.make %d ' ' in really_input inx s 0 %d; s@." digest_len digest_len;
      fprintf buf "let output_magic = output_string@.";
      fprintf buf "let cache_magic = \"%s\"@." (digest_files cache_files);
      fprintf buf "let env_magic = \"%s\"@."   (digest_files env_files);
      fprintf buf "let ir_magic = \"%s\"@."    (digest_files ir_files);
      fprintf buf "let obj_magic = \"%s\"@."   (digest_files obj_files);
      fprintf buf "let lib_dir = \"%s\"@." (String.escaped s);
      fprintf buf "let version = \"%s\"@." (String.escaped version);
      fprintf buf "let version_message = \"OMake %s:\\n\\tbuild [%s %s %d %02d:%02d:%02d %d]\\n\\ton %s%s%s\"@."
         version
         wday_names.(wday)
         mon_names.(mon)
         mday
         hour
         min
         sec
         (year + 1900)
         (Unix.gethostname ())
         "\n\nLibrary directory: " (String.escaped s)

(************************************************************************
 * OMakeroot file.
 *)
let omake_root buf name =
   let version = read_version () in
   let inx = open_in name in
      fprintf buf "@[<v 0>#@ # Required version of omake@ #@ OMakeVersion(%s, %s)@ @ " version version;
      copy_file buf inx;
      fprintf buf "@]@.";
      close_in inx

(************************************************************************
 * Main function.
 *)
let make_magic = ref false
let make_root = ref None
let output_file = ref None

let spec =
   ["--magic", Arg.Set make_magic, "generate the omake_magic.ml file";
    "--root",  Arg.String (fun s -> make_root := Some s),  "generate the OMakeroot file";
    "-o", Arg.String (fun s -> output_file := Some s), "set the output file"]

let usage = "Generate special files"

let anon s =
   raise (Invalid_argument ("unknown option" ^ s))

let main () =
   Arg.parse spec anon usage;
   let buf =
      match !output_file with
         Some name ->
            open_out name
       | None ->
            raise (Invalid_argument "use the -o option to specify an output file")
   in
      if !make_magic then
         omake_magic buf
      else
         match !make_root with
            Some name ->
               omake_root buf name
          | None ->
               Arg.usage spec usage;
               exit 2

let _ =
   Printexc.catch main ()

(*!
 * @docoff
 *
 * -*-
 * Local Variables:
 * Caml-master: "compile"
 * End:
 * -*-
 *)
